/*
 *   OSPFD routing daemon
 *   Copyright (C) 1998 by John T. Moy
 *   
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License
 *   as published by the Free Software Foundation; either version 2
 *   of the License, or (at your option) any later version.
 *   
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *   
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

extern char *LOOPADDR;

class SimPktQ;

class SimSys : public Linux {
    int ctl_fd;     // Connection to controller
    TcpPkt ctl_pkt; // Packet processing for controller connection
    int uni_fd;	// Connection for packets addressed to us
    uns16 uni_port; // Our port for locally addressed packets
    int ticks; // Elapsed time in simulated ticks
    AVLtree address_map; // IP address to group mapping
    AVLtree port_map; // Phyint to file descriptor mapping
    AVLtree membership; // Interface group membership		   
    SimPktQ *xmt_head;  // Queued transmissions
    SimPktQ *xmt_tail;
    bool ipforwarding; // Whether IP forwarding is enabled
  public:
    SimSys(int fd);
    ~SimSys();
    
    void sendpkt(InPkt *pkt, int phyint, InAddr gw=0);
    void sendpkt(InPkt *pkt);
    bool phy_operational(int phyint);
    void phy_open(int phyint);
    void phy_close(int phyint);
    void join(InAddr group, int phyint);
    void leave(InAddr group, int phyint);
    void ip_forward(bool enabled);
    void enable_multicast_routing(int phyint);
    void disable_multicast_routing(int phyint);
    void rtadd(InAddr, InMask, MPath *, MPath *, bool); 
    void rtdel(InAddr, InMask, MPath *);
    void add_mcache(InAddr src, InAddr group, int n_in,
		    int *in_phyints, int n_out, int *out_phyints);
    void del_mcache(InAddr src, InAddr group);
    char *phyname(int phyint);
    void sys_spflog(int code, char *buffer);
    void halt(int code, char *string);

    void process_uni_fd();
    void local_demux(int phyint, InPkt *pkt, int plen);
    uns16 get_port_addr(InAddr, InAddr &home);
    int get_fd(int phyint);
    void sendqueue();
    void send_multicast(byte *data, size_t len);
    void recv_ctl_command();
    void config(int type, int subtype, void *msg);
    void send_tick_response();

    friend int main(int argc, char *argv[]);
};

/* Map an IP address of a simulated router to the
 * port that router will be listening to.
 */

class AddressMap : public AVLitem {
    uns16 port;
    InAddr home;
  public:
    AddressMap(InAddr, InAddr);
    friend class SimSys;
};

/* Map a phyint to a data structure describing
 * the interface.
 */

class PhyintMap : public AVLitem {
    bool working;
    bool promiscuous;
    char name[8];
  public:
    PhyintMap(int phyint);
    friend class SimSys;
};

// Maximum size of an IP packet
const int MAX_IP_PKTSIZE = 65535;

/* Stored packet forwarding info.
 * Packets queued until timer tick
 */

struct XmtInfo {
    int phyint;
    InAddr nh;
};

/* Class for queueing control packets.
 */

class SimPktQ {
    SimPktQ *next;
    XmtInfo hdr;
    byte *ip_data;
  public:
    friend class SimSys;
};
