/*
 *   OSPFD routing simulator
 *   Copyright (C) 1999 by John T. Moy
 *   
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License
 *   as published by the Free Software Foundation; either version 2
 *   of the License, or (at your option) any later version.
 *   
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *   
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

const int MON_PORT = 13000;	// Initial monitoring port
const int NET_PORT = 14000;	// Initial network segment port
const int TICKS_PER_SECOND = 20; // Simulated time granularity

/* Packet types exchanged between the simulation
 * controller and the individual ospfd simulations.
 */

enum {
    // From controller to simulated ospfds
    SIM_TICK = 1,	// Simulated time tick
    SIM_CONFIG,		// Config messages
    SIM_CONFIG_DEL,	// Config deletion messages
    SIM_ADDRMAP,	// Address to group mappings

    // Responses from ospfds
    SIM_HELLO = 100,	// Initial identification
    SIM_TICK_RESPONSE,  // Tick response
    SIM_LOGMSG,		// Logging message
};

/* Body of the address map message is an array of the
 * following structures which map unicast IP addresses
 * into ports listening for the address.
 */

struct AddrMap {
  InAddr addr;		// Router interface address
  uns32 port;		// Port listening for address
  InAddr home;		// Router ID of owning router
};

/* Body of the Hello message carries the OSPF router ID
 * of the simulated router.
 */

struct SimHello {
    InAddr rtrid;
    uns16 myport;
};

/* Tick responses carry the #LSAs and checksum for 
 * AS-externals and each area, so that the controller
 * can tell whether the routers' databases are
 * sychronized.
 */

struct DBStats {
    uns32 n_exlsas;	// # AS-external-LSAs in database
    uns32 ex_dbxsum;	// AS-external-LSA checksum
    InAddr area_id;
    uns32 n_lsas;	// # LSAs in database
    uns32 dbxsum;	// Area database checksum
};
