/*
 *   OSPFD routing daemon
 *   Copyright (C) 1998 by John T. Moy
 *   
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License
 *   as published by the Free Software Foundation; either version 2
 *   of the License, or (at your option) any later version.
 *   
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *   
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/* Routines dealing with group-membership-LSAs.
 * This includes originating, parsing, and
 * running routing calculations.
 */

#include "ospfinc.h"

/* Constructor for a group-membership-LSA.
 */

grpLSA::grpLSA(SpfArea *a, LShdr *hdr, int blen) : LSA(a, hdr, blen)

{
}

/* Parse a group-membership-LSA. Just set the exception
 * flag, so that the caller will instead store
 * the body of the LSA.
 */

void grpLSA::parse(LShdr *)

{
    exception = true;
}

/* Unparse a group-LSA. NULL function.
 */

void grpLSA::unparse()

{
}

/* Build a group-membership-LSA. Since the parse function
 * always sets the exception flag, this function is never really
 * called.
 */

void grpLSA::build(LShdr *)

{
}

/* Reoriginate a group-membership-LSA.
 */

void grpLSA::reoriginate(int forced)

{
    lsa_ap->grp_orig(ls_id(), forced);
}

/* Build the group-membership-LSA for a given multicast group
 * and area.
 */

void SpfArea::grp_orig(InAddr /* group */, int /* forced */)

{
}

/* Determine whether there are grpup members directly
 * attached to a transit node, by looking for
 * the appropriate group-membership-LSA referencing
 * the transit node.
 * The body of the group-membership-LSA has not been
 * parsed, and is still in network-byte order.
 */

bool    TNode::has_members(InAddr group)

{
    grpLSA *glsa;
    size_t len;
    GMref *vp;

    glsa = (grpLSA *) ospf->FindLSA(lsa_ap, LST_GM, group, adv_rtr());
    if (!glsa)
	return(false);
    // Search for vertex
    len = glsa->ls_length() - sizeof(LShdr);
    vp =  (GMref *) glsa->lsa_body;
    for (; len >= sizeof(*vp); vp++, len -= sizeof(*vp)) {
	if (ntoh32(vp->ls_id) != ls_id())
	    continue;
	if (ntoh32(vp->ls_type) == ls_type())
	    return(true);
    }

    return(false);
}

